#include "v3.h"
#include "m33.h"


V3::V3(float x, float y, float z) {

  xyz[0] = x;
  xyz[1] = y;
  xyz[2] = z;

}

// cross product
V3 V3::operator^(V3 op1) {

  V3 ret;
  ret.xyz[0] = xyz[1]*op1.xyz[2]-xyz[2]*op1.xyz[1];
  ret.xyz[1] = xyz[2]*op1.xyz[0]-xyz[0]*op1.xyz[2];
  ret.xyz[2] = xyz[0]*op1.xyz[1]-xyz[1]*op1.xyz[0];

  return ret;

}

float& V3::operator[](int i) {

  return xyz[i];

}

float V3::operator*(V3 op1) {

  V3 &op0 = *this;
  return 
    op0[0]*op1[0] + 
    op0[1]*op1[1] + 
    op0[2]*op1[2];

}

float V3::Length() {

  V3 &op0 = *this;
  return sqrtf(op0*op0);

}

ostream& operator<<(ostream &ostr, V3 v) {

  ostr << v[0] << " " << v[1] << " " << v[2];
  return ostr;

}

V3 V3::operator+(V3 op1) {

  V3 ret, &op0 = *this;
  ret[0] = op0[0] + op1[0];
  ret[1] = op0[1] + op1[1];
  ret[2] = op0[2] + op1[2];
  return ret;

}

V3 V3::operator-(V3 op1) {

  V3 ret, &op0 = *this;
  ret[0] = op0[0] - op1[0];
  ret[1] = op0[1] - op1[1];
  ret[2] = op0[2] - op1[2];
  return ret;

}

V3 V3::operator/(float scf) {

  V3 ret, &v = *this;
  ret[0] = v[0] / scf;
  ret[1] = v[1] / scf;
  ret[2] = v[2] / scf;
  return ret;
}
